/*

Library for PCF8583 version 1.5.0.0 (Hardware)


      The offset from Y2000 is stored in the PCF8583 RAM at address 0x10.
      The time format is 24h and weekdays are not used.
      
  - struct TTime
  - char const weekday_names[7][10]
  - unsigned char RTC_Read(unsigned char addr)
  - void RTC_Write(unsigned char addr, unsigned char value
  - void Init_TimeRW_Default_Value()
  - void WriteTime()
  - void ReadTime()
  
  P.S :
  - void RTC_Clock_Out_1Hz() ne fonctionne pas en mode Hardware ( I2C1_ ) mais fonctionne en mode Soft ( Soft_I2C_ ) dans la simulation Proteus
*/

struct TTime
{
  char year, month, day, hours, minutes, seconds;
} TimeRW;

char yearmod4, byteRead;

// RTC Definitions (PCF8583)
#define adresse_PCF8583_Writing   0xA0    // address PCF8583 for Writing R/W=0
#define adresse_PCF8583_Reading   0xA1    // address PCF8583 for Reading R/W=1

// Register addresses
#define PCF8583_CTRL_STATUS_REG   0x00
#define PCF8583_100S_REG          0x01
#define PCF8583_SECONDS_REG       0x02
#define PCF8583_MINUTES_REG       0x03
#define PCF8583_HOURS_REG         0x04
#define PCF8583_DATE_REG          0x05
#define PCF8583_MONTHS_REG        0x06
#define PCF8583_TIMER_REG         0x07
#define PCF8583_ALARM_CONTROL_REG 0x08
#define PCF8583_ALARM_100S_REG    0x09
#define PCF8583_ALARM_SECS_REG    0x0A
#define PCF8583_ALARM_MINS_REG    0x0B
#define PCF8583_ALARM_HOURS_REG   0x0C
#define PCF8583_ALARM_DATE_REG    0x0D
#define PCF8583_ALARM_MONTHS_REG  0x0E
#define PCF8583_ALARM_TIMER_REG   0x0F
// Use the first NVRAM address for the year byte.
#define PCF8583_YEAR_REG          0x10
// Commands for the Control/Status register.
#define PCF8583_START_COUNTING    0x00
#define PCF8583_STOP_COUNTING     0x80

char const weekday_names[7][10] =
{
 {"Sunday"},
 {"Monday"},
 {"Tuesday"},
 {"Wednesday"},
 {"Thursday"},
 {"Friday"},
 {"Saturday"}
};

/**************************************************************************************************
* Read data from RTC PCF8583
* input : address of RTC register
* output: value of of RTC register
**************************************************************************************************/
unsigned char RTC_Read(unsigned char addr)
{
  unsigned char value;

  I2C1_Start();                   // Issue start signal
  I2C1_Wr(adresse_PCF8583_Writing);              // Address PCF8583, see PCF8583 datasheet
  I2C1_Wr(addr);                  // Start from address 2
  I2C1_Start();                   // Issue repeated start signal
  I2C1_Wr(adresse_PCF8583_Writing);              // Address PCF8583 for reading R/W=1

  value = I2C1_Rd(0);             // Read seconds byte
  I2C1_Stop();                    // Issue stop signal

  return value;
}

/**************************************************************************************************
* Write data from RTC PCF8583
* input : address of RTC register, value of of RTC register
**************************************************************************************************/
void RTC_Write(unsigned char addr, unsigned char value)
{
  I2C1_Start();         // Issue start signal
  I2C1_Wr(adresse_PCF8583_Writing);    // Address PCF8583
  I2C1_Wr(addr);        // Start from address
  I2C1_Wr(value);       // Write value to RTC register
  I2C1_Stop();          // Issue stop signal
}

void Init_TimeRW_Default_Value()
{
 /*
 TimeRW.year    = 7;    // 2007 (offset from 2000)
   TimeRW.month   = 12;
   TimeRW.day     = 31;
   TimeRW.hours   = 23;
   TimeRW.minutes = 59;
   TimeRW.seconds = 40;
   */
   
   // P.S : utilisation du programme BuildVersionGen  : http://www.libstock.com/projects/view/935/build-version-generator
   TimeRW.year    = BuildYear - 2000;    // (offset from 2000)
   TimeRW.month   = BuildMonth;
   TimeRW.day     = BuildDay;
   TimeRW.hours   = BuildHour;
   TimeRW.minutes = BuildMinute;
   TimeRW.seconds = BuildSecond;
}

void WriteTime()
{
   I2C1_Start();            // issue start signal
   I2C1_Wr(adresse_PCF8583_Writing);          // address PCF8583
   I2C1_Wr(0);            // start from word at address 0 (configuration word)
   I2C1_Wr(PCF8583_STOP_COUNTING); //0x80);          // write 0x80 to config. (pause counter...)
   I2C1_Wr(0);            // write 0 to cents word
   I2C1_Wr(((TimeRW.seconds/10)<<4) + (TimeRW.seconds%10)); // write seconds word
   I2C1_Wr(((TimeRW.minutes/10)<<4) + (TimeRW.minutes%10)); // write minutes word
   I2C1_Wr(((TimeRW.hours/10)<<4) + (TimeRW.hours%10)); // write hours word
   I2C1_Wr(((TimeRW.year%4)<<6) + ((TimeRW.day/10)<<4) +(TimeRW.day%10));
   I2C1_Wr(((TimeRW.month/10)<<4) + (TimeRW.month%10));  // write weekday/month
   I2C1_Stop();             // issue stop signal

   I2C1_Start();            // issue start signal
   I2C1_Wr(adresse_PCF8583_Writing);          // address PCF8583
   I2C1_Wr(PCF8583_YEAR_REG);          // start from word at address 0x10
   I2C1_Wr(TimeRW.year);         // write year to RAM
   I2C1_Stop();             // issue stop signal

   I2C1_Start();            // issue start signal
   I2C1_Wr(adresse_PCF8583_Writing);          // address PCF8583
   I2C1_Wr(0);            // start from word at address 0
   I2C1_Wr(PCF8583_START_COUNTING); //);            // write 0 to config word (enable counting)
   I2C1_Stop();             // issue stop signal
}

void ReadTime()
{
 char updateYear;

   updateYear = 0;

   I2C1_Start();                // issue start signal
   I2C1_Wr(adresse_PCF8583_Writing);               // address PCF8583
   I2C1_Wr(2);                  // first word address
   I2C1_Repeated_Start();       // issue repeated start signal
   I2C1_Wr(adresse_PCF8583_Reading);               // address PCF8583 for reading R/W=1

   byteRead = I2C1_Rd(1);       // read seconds byte
   TimeRW.seconds = (byteRead >> 4)*10 + (byteRead & 0x0F); // transform seconds
   //while(I2C1_Is_Idle() == 0)  asm nop;    // this lines are not neccessary in new versions
                      // starting from mikroC 6.0.0.1

   byteRead = I2C1_Rd(1);     // read minutes byte
   TimeRW.minutes = (byteRead >> 4)*10 + (byteRead & 0x0F); // transform minutes
   //while(I2C1_Is_Idle() == 0)  asm nop;

   byteRead = I2C1_Rd(1);     // read hours byte
   TimeRW.hours = (byteRead >> 4)*10 + (byteRead & 0x0F); // transform hours
   //while(I2C1_Is_Idle() == 0)  asm nop;

   byteRead = I2C1_Rd(1);     // read year/day byte
   TimeRW.day = ((byteRead & 0b00110000) >> 4)*10 + (byteRead & 0x0F); // transform day
   yearmod4 = (byteRead & 0b11000000) >> 6;   // get year mod 4 from RTC
   //while(I2C1_Is_Idle() == 0)  asm nop;

   byteRead = I2C1_Rd(0);     // read weekday/month byte
   TimeRW.month = ((byteRead & 0b00010000) >> 4)*10 + (byteRead & 0x0F); // transform month
   //while (I2C1_Is_Idle() == 0) asm nop;
   I2C1_Stop();

   I2C1_Start();                 // issue start signal
   I2C1_Wr(adresse_PCF8583_Writing);               // address PCF8583
   I2C1_Wr(PCF8583_YEAR_REG);               // first word address
   I2C1_Repeated_Start();        // issue repeated start signal
   I2C1_Wr(adresse_PCF8583_Reading);               // address PCF8583 for reading R/W=1

   byteRead = I2C1_Rd(0);     // read year
   if(yearmod4 != byteRead % 4 )
   { // check if year is incremented in RTC
       byteRead++;  // in this case the new value should be written to RTC RAM at address 16 (0x10)
       updateYear = 1;
   }
   TimeRW.year = byteRead;

   //while (I2C1_Is_Idle() == 0)  asm nop;
   I2C1_Stop();

   if(updateYear > 0)
   {
      I2C1_Start();            // issue start signal
      I2C1_Wr(adresse_PCF8583_Writing);          // address PCF8583
      I2C1_Wr(PCF8583_YEAR_REG);          // start from word at address 0x10
      I2C1_Wr(TimeRW.year);         // write year to RAM
      I2C1_Stop();             // issue stop signal
    }
}

/**************************************************************************************************
* PCF8583 Functions
**************************************************************************************************/

void RTC_Clock_Out_1Hz()
{
 /*
 I2C1_Start();
 I2C1_Wr(adresse_PCF8583_Writing);
 I2C1_Wr(PCF8583_ALARM_DATE_REG); //0x0D);
 I2C1_Wr(0x83);
 I2C1_Stop();
 */
 RTC_Write(PCF8583_ALARM_DATE_REG, 0x83);
}